'	Program :	Days
	Author  :	Harald Piske
	Written :	03/2001
	Purpose :	understand user input (huh!)

	History :
';

sub future			# read a date that has to be in the future
{					# basic format:  m/d/y  h:m:s   or   +days  h:m:s
					# this function tries to make sense of any input, eg:
					# 17				5pm today or tomorrow if now > 5pm
					# 17:15				5.15pm today or tomorrow
					# 17.15				same
					# 5.15p				same
					# 17,15				3pm on the 17th (this month or next)
					# 3/17 17.15		5.15p on mar 17
					# 28.4.2001			d. or d.m or d.m. or d.m.y
					# 4/28/2001			d (with time) or m/d or m/d/y
					# 2001-4-28			m-d or y-m-d
					# +24				24hrs from now
					# +2.5				2hrs, 30mins from now
					# +2,5				2days, 5hrs from now
			# this routine is not fool-proof. Irrational input like
			# 17.3/2001, 00:30p   or   +3/17
			# will result in creative output.
	local ($_, $add, $now, $then, $more, $gmdiff, @dat, @now, @then);
	$_ = shift;
	$add = s/^\s*\+\s*//;					# strip leading + sign
	return '' unless @_ = m!^\s*(([-\d/\.]+)[,\s]+[-\s]*)?(\d+|\d\S*\d)([amp\.]*)\s*$!i;
											# split into date, time value and am/pm
	@now = $_[2] =~ /\d+/g;
	$now < length $_ and $now = length $_ for @now;
	@_ = /(\s*)(\S*)/ if $_[0] eq '' &&
		(@now > 2 || $now > 2 || $_[3] eq '.' || $_[2] =~ m!/!);
				# three numbers or any one more than two digits: it's  a date.
				# Single number is hour, but single number followed by a dot,
				# or several numbers delimited by dots or slashes are a date.
	@dat = grep $_, $_[1] =~ /\d*/g;				# get numbers of date
	@dat = reverse @dat if $_[1] =~ /\./;			# d.m(.y) -> (y)md
	@dat = @dat[2,0,1] if $_[1] =~ m!\d/\d+/\d!;	# m/d/y -> ymd
	unshift @dat, 0 while $#dat < 2;				# d or md -> ymd
	push @dat, $_[2] =~ /(\d*)\D*/g;				# add time values
	$#dat = 5;										# ymdh or ymdhM -> ymdhMs
	$dat[3] += 12 if $dat[3] < 12 && $_[3] =~ /p/i;	# 1p .. 11p -> 13h .. 23h
	$dat[3] = 0 if $dat[3] == 12 && $_[3] =~ /a/i;	# 12a -> 00h
	@dat = reverse @dat;							# sMhdmy
	@now = localtime ($now = time);
	$gmdiff = seconds (gmtime $now) - seconds (@now);
	$now[4]++;										# month comes as 0..11
	$now[5] += 1900;								# year as offset from 1900
	if ($add)		# (that's 2 digits, with millenium issue simply ignored)
	{
		@then = @now;								# relative to now:
		$then[$_] += $dat[$_] for (0..3);			# add d,h,M, and s
	}else{
		@then = @dat;								# absolute:
		$then[$_] ||= $now[$_] for (3..5);			# get current values for
	}												# d,m,y where not specified
	$then[4]--;										# return month to 0..11
	return $then unless $then = seconds (@then);	# intelligible?
	$now -= $gmdiff;								# compare UTCs
	while (!$add && $then <= $now)
	{
		@_ = @then;
		unless ($dat[3]++)							# point in the past and
		{											# no day specified?
			$_[3]++;								# y: try tomorrow
			next;
		}
		unless ($dat[4]++)							# (still) in the past and
		{											# no month specified?
			$_[5]++, $_[4] = 0 if ++$_[4] > 11;		# try this day next month
			next;									# consider 12/x -> 1/x+1
		}
		last if $dat[5]++;							# (still) in the past?
		$_[5]++;									# maybe next year.
	}continue{
		$then = seconds (@_);						# recalculate
	}
	return $then + gmdiff;							# return local time
}
############################################################################
sub seconds											# counterpart to gmtime
{
	local $_ = $_[5];								# year
	$_ += 2000 if $_ < 50;
	$_ += 1900 if $_ < 200;
	return '' if $_ < 1900 || $_ > 2099;			# cannot determine
	$_ -= 1968;										# must divide by 4
	$_ = 365 * ($_ - 2) + int ($_ / 4);
	$_-- unless $_[5] & 3 || $_[4] > 1;	# leap-year not relevant for jan,feb
	$_ += (-1,30,58,89,119,150,180,211,242,272,303,333)[$_[4]] + $_[3];
	return $_[0] + 60 * $_[1] + 3600 * $_[2] + 86400 * $_;
}
sub date
{
	local $_ = shift;
	local $dec = 1900;
	while ($_ < 0)
	{
		$_ += 20 * 365.25 * 86400;
		$dec -= 20;
	}
	while ($_ > 60 * 365.25 * 86400)
	{
		$_ -= 20 * 365.25 * 86400;
		$dec += 20;
	}
	@_ = gmtime $_;
	$_[5] += $dec;
	$_[4]++;
	return sprintf "%02d.%02d.%04d %02d:%02d:%02d", @_[3..5,2,1,0];
}
push @ARGV, '+0' unless @ARGV;
$_ = $date = future shift;
$_ = $date = /^\+/ ? $date + $_ * 86400 : future ($_) for @ARGV;
exit print 'Days.pl - (w) 04/2001 by H. Piske
A study in interpreting user input - namely dates.
Enter one or more dates any which way you like and see what happens.
Enter +x to add x days to the previous value.
If you must, include a time value, seperated from the date by a comma.
' unless @ARGV = grep $_ ne '', $_, @ARGV;
print date ($_), ': ', int ($_ / 86400), " days, $_ seconds\n" for @ARGV;
while (($now = shift) ne '')
{
	print date ($now), " to ", date ($_), ": ",
		int (($dif = abs ($now - $_)) / 86400),
		" days, $dif seconds\n" for @ARGV;
}
